/**
 * @file StereoAmp_click_AVR.c
 * @brief <h2> StereoAmp click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * StereoAmp click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        3 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    ATMEGA32
 * -<b> Dev. Board    </b> :    EasyAVR v7
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    StereoAmp click board
 * -<b> SW            </b> :    mikroC PRO for AVR v6.1.2
 *
 */

/******************************************************************************
* Includes
*******************************************************************************/
#include "StereoAmp.h"

/******************************************************************************
* Variable Definitions
*******************************************************************************/
// LCD module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

uint8_t volume_control = 0;
uint8_t volume_temp = 0;

const char custom_char_1[] = {16,16,16,16,16,16,16,16};
const char custom_char_2[] = {24,24,24,24,24,24,24,24};
const char custom_char_3[] = {28,28,28,28,28,28,28,28};
const char custom_char_4[] = {30,30,30,30,30,30,30,30};
const char custom_char_5[] = {31,31,31,31,31,31,31,31};

/******************************************************************************
* Public Functions
*******************************************************************************/
void Generate_Custom_Chars() {
    char i;
    Lcd_Cmd(64);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_1[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(72);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_2[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(80);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_3[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(88);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_4[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(96);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_5[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);
}


uint8_t Read_Volume_Step() {
    uint16_t adc_value = ADC_Read(5);

    // Downsample to range 0 - 31.
    adc_value &= 0x03FF;
    return adc_value / 32;
}

void Draw_Bar(unsigned row_pos, unsigned char_pos, unsigned value) {
    unsigned i;
    uint8_t white_char = 0;

    for(i = 0; i < 7; i++) {
        if (value / ((i + 1) * 5) > 0) {
            Lcd_Chr(row_pos, char_pos + i, 4);
        } else if (white_char) {
            Lcd_Chr(row_pos, char_pos + i, ' ');
        } else {
            Lcd_Chr(row_pos, char_pos + i, value % 5);
            white_char = 1;
        }
    }
}

void Write_Amp_Reg(uint8_t amp_addr, uint8_t reg_addr, uint8_t reg_data) {
    delay_ms(100);
    TWI_Start();
    TWI_Write(amp_addr);
    TWI_Write( (reg_addr & REG_ADDR_MASK) | (reg_data & DATA_MASK) );
    TWI_Stop();
}


void Init() {
    // LCD initialisation.
    Lcd_Init();
    Lcd_Cmd(_LCD_CLEAR);               // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off

    Generate_Custom_Chars();
    
    volume_control = Read_Volume_Step();
    Draw_Bar(1, 10, volume_control);
    Lcd_Out(1, 2, gain_text[volume_control]);

    // StereoAmp initialisation.

    // IMPORTANT: Toggle SDA and SCL lines, before initialising TWI module to enable amplifiers.
    DDC0_bit = 1;
    DDC1_bit = 1;
    
    PORTC0_bit = 1;
    PORTC1_bit = 1;
    
    Delay_ms(300);
    
    PORTC0_bit = 0;
    PORTC1_bit = 0;
    
    Delay_ms(300);

    TWI_Init(I2C_CLK);
    Delay_ms(800);

    // Power On and Input 1 enabled.
    Write_Amp_Reg(LEFT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);
    Write_Amp_Reg(RIGHT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);

    // Set volume.
    Write_Amp_Reg(LEFT_SPEAKER, VOLUME_CONTROL1_REG, volume_control);
    Write_Amp_Reg(RIGHT_SPEAKER, VOLUME_CONTROL1_REG, volume_control);

    // Fault flags.
    Write_Amp_Reg(LEFT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);
    Write_Amp_Reg(RIGHT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);

    // Enable diagnostic.
    Write_Amp_Reg(LEFT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);
    Write_Amp_Reg(RIGHT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);
}


void Set_Volume(char amp_addr, char value) {
    Write_Amp_Reg(amp_addr, VOLUME_CONTROL1_REG, value);
}


void main() {
    Init();

    while(1) {
        volume_temp = Read_Volume_Step();

        if(volume_temp != volume_control) {
            volume_control = volume_temp;

            Draw_Bar(1, 10, volume_control);
            Lcd_Out(1, 2, gain_text[volume_control]);

            Set_Volume(LEFT_SPEAKER, volume_control);
            Set_Volume(RIGHT_SPEAKER, volume_control);
        }

        Delay_ms(POLLING_PERIOD);
    }
}
/*************** END OF FUNCTIONS *********************************************/